package vn.lecle.lecle_volume_flutter

import android.app.Activity
import android.app.NotificationManager
import android.content.Context
import android.content.Intent
import android.media.AudioManager
import android.os.Build
import android.provider.Settings
import androidx.annotation.NonNull
import androidx.core.content.ContextCompat.startActivity
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result


/** LecleVolumeFlutterPlugin */
class LecleVolumeFlutterPlugin: FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel
  private lateinit var activity: Activity
  private var audioManager: AudioManager? = null
  private var streamType = 0
  private lateinit var context : Context

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "lecle_volume_flutter")
    channel.setMethodCallHandler(this)
    context = flutterPluginBinding.applicationContext
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    when (call.method) {
        "initVolumeStream" -> {
          val i = call.argument<Int>("streamType")!!
          streamType = i
          initVolumeStream(i)
          result.success(null)
        }
        "getMaxVol" -> {
          result.success(getMaxVol())
        }
        "getVol" -> {
          result.success(getVol())
        }
        "setVol" -> {
          val i = call.argument<Int>("newVol")!!
          val showUiFlag = call.argument<Int>("showVolumeUiFlag")!!
          setVol(i, showUiFlag)
          result.success(0)
        }
        else -> {
          result.notImplemented()
        }
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activity = binding.activity
  }

  override fun onDetachedFromActivityForConfigChanges() {
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
  }

  override fun onDetachedFromActivity() {
  }

  private fun initVolumeStream(i: Int) {
    activity.volumeControlStream = i
  }

  private fun initAudioManager() {
    audioManager = activity.applicationContext.getSystemService(Context.AUDIO_SERVICE) as AudioManager
  }

  private fun getMaxVol(): Int {
    initAudioManager()
    return audioManager!!.getStreamMaxVolume(streamType)
  }

  private fun getVol(): Int {
    initAudioManager()
    return audioManager!!.getStreamVolume(streamType)
  }

  private fun setVol(i: Int, showVolumeUiFlag: Int): Int {
    initAudioManager()

    audioManager!!.setStreamVolume(streamType, i, showVolumeUiFlag)
    return audioManager!!.getStreamVolume(streamType)
  }

  // https://stackoverflow.com/questions/39151453/in-android-7-api-level-24-my-app-is-not-allowed-to-mute-phone-set-ringer-mode
  private fun checkDoNotDisturbPermission(): Boolean {
    val notificationManager : NotificationManager = context.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M
      && !notificationManager.isNotificationPolicyAccessGranted
    ) {
      val intent = Intent(
        Settings.ACTION_NOTIFICATION_POLICY_ACCESS_SETTINGS
      )
      intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
      startActivity(context, intent, null)
      return false
    }

    return true
  }
}
