import Flutter
import UIKit
import MediaPlayer

public class SwiftLecleVolumeFlutterPlugin: NSObject, FlutterPlugin {
    var streamType = StreamType.system
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "lecle_volume_flutter", binaryMessenger: registrar.messenger())
        let instance = SwiftLecleVolumeFlutterPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch (call.method) {
        case "initVolumeStream":
            if let args = call.arguments as? Dictionary<String, Any>,
               let i = args["streamType"] as? Int {
                streamType = initVolumeStream(i: i)
                result(nil)
            } else {
                result(FlutterError.init(code: "Argument not provided", message: nil, details: nil))
            }
        case "getMaxVol":
            result(getMaxVol())
        case "getVol":
            result(getVol())
        case "setVol":
            if let args = call.arguments as? Dictionary<String, Any>,
               let i = args["newVol"] as? Double,
               let showUIFlag = args["showVolumeUiFlag"] as? Int {
                setVol(i: i, showVolumeUiFlag: showUIFlag)
                result(0)
            }
        default:
            result(FlutterMethodNotImplemented)
        }
    }
    
    private func initVolumeStream(i: Int) -> StreamType {
        switch i {
        case 0:
            return StreamType.inCall
        case 1:
            return StreamType.system
        case 2:
            return StreamType.ring
        case 3:
            return StreamType.media
        case 4:
            return StreamType.alarm
        case 5:
            return StreamType.notifications
        default:
            return StreamType.system
        }
    }
    
    private func getMaxVol() -> Double {
        // iOS volume between 0.0 -> 1.0
        return 1.0
    }
    
    private func getVol() -> Double {
        let audioSession = AVAudioSession.sharedInstance()
        var volume: Float = 0.0
        do {
            try audioSession.setActive(true)
            volume = audioSession.outputVolume
        } catch {
            print("Error Setting Up Audio Session")
        }
        
        return Double(volume)
    }
    
    private func setVol(i: Double, showVolumeUiFlag: Int) {
        MPVolumeView.setVolume(Float(i), showVolumeUiFlag: showVolumeUiFlag)
    }
}

// Update system volume (https://stackoverflow.com/questions/33168497/ios-9-how-to-change-volume-programmatically-without-showing-system-sound-bar-po)
extension MPVolumeView {
    static func setVolume(_ volume: Float, showVolumeUiFlag: Int) {
        let volumeView = MPVolumeView(frame: .zero)
        volumeView.showsVolumeSlider = showVolumeUiFlag == 1 ? true : false
        let slider = volumeView.subviews.first(where: { $0 is UISlider }) as? UISlider
        
        DispatchQueue.main.asyncAfter(deadline: DispatchTime.now() + 0.01) {
            slider?.value = volume
        }
    }
}

enum StreamType {
    case inCall, system, ring, media, alarm, notifications
}
